# This file was auto-generated by Shut. DO NOT EDIT
# For more information about Shut, check out https://pypi.org/project/shut/

from __future__ import print_function
import io
import os
import setuptools
import sys

readme_file = 'README.md'
if os.path.isfile(readme_file):
  with io.open(readme_file, encoding='utf8') as fp:
    long_description = fp.read()
else:
  print("warning: file \"{}\" does not exist.".format(readme_file), file=sys.stderr)
  long_description = None

requirements = [
  'click',
  'pyyaml',
]

setuptools.setup(
  name = 'pyvboxmanage',
  version = '0.3.0',
  author = 'Nicholas de Jong',
  author_email = 'me@nicholasdejong.com',
  description = 'Wrapper tool around VBoxManage that facilitates the orchestration of VBoxManage commands from a simple YAML configuration file that matches the input opts/args for VBoxManage.',
  long_description = long_description,
  long_description_content_type = 'text/markdown',
  url = 'https://pyvboxmanage.readthedocs.io',
  license = 'BSD2',
  packages = setuptools.find_packages('src', ['test', 'test.*', 'tests', 'tests.*', 'docs', 'docs.*']),
  package_dir = {'': 'src'},
  include_package_data = True,
  install_requires = requirements,
  extras_require = {},
  tests_require = [],
  python_requires = '>=3.5.0,<4.0.0',
  data_files = [],
  entry_points = {
    'console_scripts': [
      'pyvboxmanage = pyvboxmanage.cli.entrypoints:pyvboxmanage',
    ]
  },
  cmdclass = {},
  keywords = ['virtuabox', 'vbox', 'vboxmanage', 'devops'],
  classifiers = ['Environment :: Console', 'Intended Audience :: Developers', 'Intended Audience :: Information Technology', 'Programming Language :: Python :: 3.5', 'Programming Language :: Python :: 3.6', 'Programming Language :: Python :: 3.7', 'Programming Language :: Python :: 3.8', 'Programming Language :: Python :: 3.9', 'License :: OSI Approved :: BSD License'],
  zip_safe = True,
)
